from pydantic import BaseModel


class AiPayloadMediaInfo(BaseModel):
    description: str
    tags: list[str]
    filename: str
    text: list[str]

    def __str__(self):
        return f"Description: {self.description}, Tags: {self.tags}, Filename: {self.filename}, Text: {self.text}"

    class Config:
        extra = "ignore"




from pathlib import Path

import typer

from pylizlib.core.app.configini import CfgPath
from pylizlib.core.app.pytoml import PyProjectToml
from pylizlib.core.os.path import PathMatcher

app = typer.Typer(help="General utility scripts.")


@app.command("expFileList")
def exp_file_list(
        input: Path = typer.Argument(..., help="Input path to scan"),
        output: Path = typer.Argument(..., help="Output path where to save the file"),
        file_name: str = typer.Argument(..., help="Name of the file to save"),
        recursive: bool = typer.Option(False, "--recursive", help="Enable recursive scan"),
):
    matcher = PathMatcher()
    matcher.load_path(input, recursive)
    matcher.export_file_list(output, file_name)


@app.command("iniDup")
def ini_dup(
        input: Path = typer.Argument(..., help="Input path to scan"),
        sections: bool = typer.Option(False, "--sections", help="Enable search for duplicate sections"),
        keys: bool = typer.Option(False, "--keys", help="Enable search for duplicate keys"),
):
    cfg = CfgPath(input)
    cfg.check_duplicates(keys, sections)


@app.command("gen-project-py")
def gen_project_py(
        pyproject_path: Path = typer.Argument(..., help="Path to the pyproject.toml file"),
        py_file: Path = typer.Argument(..., help="Path to the Python file to update"),
):
    try:
        toml = PyProjectToml(pyproject_path)
        toml.gen_project_py(py_file)
    except Exception as e:
        typer.echo(f"Error: {e}", err=True)
        raise typer.Exit(code=1)



@app.command("temp")
def temp():
    typer.echo("This is a temporary command.")


if __name__ == "__main__":
    app()


import os
import subprocess
from abc import ABC
from typing import Callable

from pylizlib.core.data import gen
from pylizlib.core.temp.pylizscript import PylizBaseAction
from pylizlib.core.os.path import check_path


class ActionGitClone(PylizBaseAction, ABC):

    def __init__(
            self,
            repo: str,
            install_dir: str,
            ignore_if_exists: bool = True,
            logs_dir: str | None = None,
            on_log: Callable[[str], None] = lambda x: None
    ):
        super().__init__()
        self.repo = repo
        self.install_dir = install_dir
        self.ignore_if_exists = ignore_if_exists
        self.logs_dir = logs_dir
        self.on_log = on_log

    def run(self):
        self.on_log("Cloning " + self.repo + "...")
        if os.path.exists(self.install_dir):
            self.on_log("Repo already installed.")
            return
        else:
            self.on_log("Repo not installed. Proceeding...")
            pathutils.check_path(self.install_dir, True)
        # Repo.clone_from(self.repo, self.install_dir)
        self.on_log("Done.")

    def reset(self):
        pass


class ActionCommandAvailable(PylizBaseAction, ABC):

    def __init__(self, command: str):
        super().__init__()
        self.command = command

    def run(self):
        status = osutils.is_command_available(self.command)
        if not status:
            raise Exception(self.command + " command not available.")

    def reset(self):
        pass


class ActionTest(PylizBaseAction, ABC):


    def __init__(self, param1: str):
        super().__init__()
        self.param1 = param1


    def run(self):
        print("test " + self.param1)

    def reset(self):
        pass


class ActionExecCli(PylizBaseAction, ABC):

    def __init__(self,
                 path: str,
                 commands: list[str],
                 path_logs: str | None = None,
                 on_log: Callable[[str], None] = lambda x: None,

    ):
        super().__init__()
        self.commands = commands
        self.on_log = on_log
        self.path_install = path
        self.path_logs = path_logs

    def run(self):
        if not os.path.isdir(self.path_install):
            raise ValueError(f"Invalid path_install: {self.path_install}")
        self.on_log("Executing: " + self.commands[0])
        output = subprocess.run(self.commands, check=True, cwd=self.path_install, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
        self.on_log("Done.")
        self.__save_output(output, self.commands[0])

    def __save_output(self, output, name: str):
        if self.path_logs is None:
            return
        check_path(self.path_logs, True)
        log_build_name = gen.gen_timestamp_log_name(f"{name}-", ".txt")
        log_build_path = os.path.join(self.path_logs, log_build_name)
        with open(log_build_path, "w") as f:
            f.write(output.stdout)
            f.write("***********************************\n")
            f.write(output.stderr)

    def reset(self):
        pass



from dataclasses import dataclass
from enum import Enum

from pylizlib.core.log.pylizLogger import logger


class QueueProgressMode(Enum):
    STEP = "Step"
    SINGLE = "Single"


@dataclass
class QueueProgressItem:
    id: str
    progress: int = 0


@dataclass
class QueueProgressStep:
    step_number: int
    progress: int = 0


@dataclass
class TaskProgress:
    task_id: str
    task_progress: int


@dataclass
class OperationProgress:
    operation_id: str
    operation_progress: float
    operation_tasks: list[TaskProgress]

    def set_task_progress(self, task_id: str, progress: int):
        for task in self.operation_tasks:
            if task.task_id == task_id:
                task.task_progress = progress
                return
        logger.warning("Task %s not found in operation %s", task_id, self.operation_id)

    def get_operation_progress(self):
        total_progress = sum(task.task_progress for task in self.operation_tasks)
        if self.operation_tasks:
            self.operation_progress = total_progress / len(self.operation_tasks)
        else:
            self.operation_progress = 0
        return self.operation_progress


class ProgressHandler:

    def __init__(self):
        self.operations: list[OperationProgress] = []

    def __get_operation_task(self, task_id: str):
        for operation in self.operations:
            for task in operation.operation_tasks:
                if task.task_id == task_id:
                    return task
        return None

    def add_operation(self, id: str, tasks_ids: list[str]):
        tasks = []
        for task_id in tasks_ids:
            tasks.append(TaskProgress(task_id=task_id, task_progress=0))
        self.operations.append(OperationProgress(
            operation_id=id,
            operation_progress=0,
            operation_tasks=tasks
        ))

    def set_task_progress(self, operation_id: str, task_id: str, progress: int):
        for operation in self.operations:
            if operation.operation_id == operation_id:
                operation.set_task_progress(task_id, progress)
                return

    def get_master_progress(self):
        total_progress = sum(operation.get_operation_progress() for operation in self.operations)
        if self.operations:
            return total_progress / len(self.operations)
        else:
            return 0

    def get_operation_progress(self, operation_id: str):
        for operation in self.operations:
            if operation.operation_id == operation_id:
                return operation.get_operation_progress()
        return 0


class QueueProgress:

    def __init__(
            self,
            mode: QueueProgressMode,
            total_count: int = 0,
            min_progress: int = 0,
            max_progress: int = 100,
    ):
        self.mode = mode
        self.total_count = total_count
        self.min_progress = min_progress
        self.max_progress = max_progress
        self.total_inner_progress = self.max_progress * total_count

        match self.mode:
            case QueueProgressMode.STEP:
                self.steps: list[QueueProgressStep] = []
                for i in range(self.total_count):
                    self.add_step(i)
            case QueueProgressMode.SINGLE:
                self.singles: list[QueueProgressItem] = []

    def add_step(self, step_number: int):
        self.steps.append(QueueProgressStep(step_number=step_number))

    def add_single(self, id: str):
        self.singles.append(QueueProgressItem(id=id))

    def set_step_progress(self, step_number: int, progress: int):
        for step in self.steps:
            if step.step_number == step_number:
                step.progress = progress
                return

    def set_single_progress(self, id: str, progress: int):
        for single in self.singles:
            if single.id == id:
                single.progress = progress
                return

    def get_step_progress(self, step_number: int):
        for step in self.steps:
            if step.step_number == step_number:
                return step.progress
        return 0

    def get_single_progress(self, id: str):
        for single in self.singles:
            if single.id == id:
                return single.progress
        return 0

    def get_total_progress(self):
        if self.total_count == 0:
            return 0
        total_progress = 0
        match self.mode:
            case QueueProgressMode.SINGLE:
                for single in self.singles:
                    total_progress += single.progress
            case QueueProgressMode.STEP:
                for step in self.steps:
                    total_progress += step.progress
        return int((total_progress / self.total_inner_progress) * 100)


import socket
from typing import Mapping

import requests


from enum import Enum

from requests.models import Response

from pylizlib.core.log.pylizLogger import logger


class NetResponseType(Enum):
    OK200 = "ok200"
    ERROR = "error"
    CONNECTION_ERROR = "connection_error"
    TIMEOUT = "timeout"
    REQUEST_ERROR = "request_error"



class NetResponse:

    def __init__(
            self,
            response: Response | None,
            response_type: NetResponseType,
            exception=None
    ):
        self.has_json_header = None
        self.json = None
        self.response = response
        self.hasResponse = self.response is not None
        if self.hasResponse:
            self.code = self.response.status_code
            self.text: str = self.response.text
        else:
            self.code = None
        self.type = response_type
        self.exception = exception
        if self.hasResponse:
            self.has_json_header = "application/json" in self.response.headers.get("Content-Type", "")
            if self.has_json_header:
                self.json = self.response.json()
        self.__log()


    def __log(self):
        logger.trace(f"NetResponse: code={self.code} | type={self.type} | jsonHeader={self.has_json_header}")

    def __str__(self):
        return ""

    def is_successful(self):
        return self.code == 200

    def is_error(self):
        return self.code != 200

    def get_error(self):
        if self.hasResponse:
            return "(" + str(self.code) + "): " + self.response.text
        else:
            pre = "(" + self.type.value + ") "
            if self.exception is not None:
                pre = pre + str(self.exception)
            return pre




HEADER_ONLY_CONTENT_JSON = {"Content-Type": "application/json"}



def test_with_head(url: str) -> bool:
    try:
        response = requests.head(url, timeout=5)
        return response.status_code < 400
    except requests.RequestException as e:
        logger.error("Error while testing URL: " + url + " - " + str(e))
        return False


def is_endpoint_reachable(url: str) -> bool:
    """Controlla se un endpoint risponde correttamente con HTTP 200."""
    try:
        response = requests.get(url, timeout=5)
        if response.status_code == 200:
            return True
        else:
            return False
    except requests.RequestException as e:
        logger.error("Error while testing URL: " + url + " - " + str(e))
        return False


def is_internet_available() -> bool:
    host = "8.8.8.8"
    port = 53
    timeout = 3
    try:
        socket.setdefaulttimeout(timeout)
        # Utilizza un blocco `with` per gestire automaticamente il socket
        with socket.create_connection((host, port)):
            return True
    except OSError:
        return False


def exec_get(
        url: str,
        headers: Mapping[str, str | bytes | None] | None = None,
        sec_timeout: int | None = 10
) -> NetResponse:
    try:
        logger.trace("Executing GET request on URL: " + url)
        response = requests.get(url, allow_redirects=True, headers=headers, timeout=sec_timeout)
        if response.status_code == 200:
            return NetResponse(response, NetResponseType.OK200)
        else:
            return NetResponse(response, NetResponseType.ERROR)
    except requests.ConnectionError as e:
        return NetResponse(None, NetResponseType.CONNECTION_ERROR, e)
    except requests.Timeout as e:
        return NetResponse(None, NetResponseType.TIMEOUT, e)
    except requests.RequestException as e:
        return NetResponse(None, NetResponseType.REQUEST_ERROR, e)


def exec_post(
        url: str,
        payload,
        headers: Mapping[str, str | bytes | None] | None = None,
        verify_bool: bool = False,
) -> NetResponse:
    try:
        logger.trace("Executing POST request on URL: " + url)
        response = requests.post(url, json=payload, verify=verify_bool, allow_redirects=True, headers=headers)
        if response.status_code == 200:
            return NetResponse(response, NetResponseType.OK200)
        else:
            return NetResponse(response, NetResponseType.ERROR)
    except requests.ConnectionError as e:
        return NetResponse(None, NetResponseType.CONNECTION_ERROR, e)
    except requests.Timeout as e:
        return NetResponse(None, NetResponseType.TIMEOUT, e)
    except requests.RequestException as e:
        return NetResponse(None, NetResponseType.REQUEST_ERROR, e)


def get_file_size_byte(url: str, exception_on_fail: bool = False) -> int:
    try:
        response = requests.head(url, timeout=5, allow_redirects=True)
        response.raise_for_status()
        file_size = response.headers.get('content-length', 0)
        if file_size is None:
            if exception_on_fail:
                raise ValueError("Unable to get file size for url: " + url)
            return -1
        return int(file_size)
    except requests.RequestException as e:
        if exception_on_fail:
            raise ValueError("Unable to get file size for url: " + url + ": " + str(e))
        return -1


from pathlib import Path
import tomllib


class PyProjectToml:
    """
    A class to handle reading and extracting information from a pyproject.toml file.
    """

    def __init__(self, toml_path: Path):
        """
        Initialize with the path to the pyproject.toml file.
        """
        self.toml_path = toml_path
        if not self.toml_path.exists():
            raise FileNotFoundError(f"File {self.toml_path} does not exist.")

    def extract_info(self) -> dict:
        """
        Extract project information from the pyproject.toml file.
        Returns a dictionary with keys: name, version, description, requires_python.
        """
        with open(self.toml_path.__str__(), "rb") as f:
            data = tomllib.load(f)

        project = data.get("project", {})
        return {
            "name": project.get("name"),
            "version": project.get("version"),
            "description": project.get("description"),
            "requires_python": project.get("requires-python")
        }

    def gen_project_py(self, path_py: Path):
        """
        Generate a Python file with project information extracted from pyproject.toml.
        """
        info = self.extract_info()
        lines = [
            f'name = {repr(info["name"])}',
            f'version = {repr(info["version"])}',
            f'description = {repr(info["description"])}',
            f'requires_python = {repr(info["requires_python"])}',
        ]
        with open(path_py.__str__(), "w", encoding="utf-8") as f:
            f.write("\n".join(lines) + "\n")


import random
import time
from collections.abc import Callable
from dataclasses import dataclass

from pylizlib.core.log.pylizLogger import logger
from pylizlib.qt.handler.operation import Operation, Task, OperationInfo, RunnerInteraction


class OperationDevDebug(Operation):

    @dataclass
    class DevDebugData:
        step_min: int = 1
        step_max: int = 100
        sleep_min: float = 0.1
        sleep_max: float = 1

    class TaskTemplate(Task):

        def __init__(self, name: str, on_task_progress_changed: Callable):
            super().__init__(name, on_task_progress_changed)
            self.data = OperationDevDebug.DevDebugData()
            self.steps = random.randint(self.data.step_min, self.data.step_max)

        def execute(self):
            for i in range(self.steps):
                progress: int = int(((i + 1) / self.steps) * 100)
                self.update_task_progress(progress)
                logger.debug(f"Log test {i}...")
                sleep = random.uniform(self.data.sleep_min, self.data.sleep_max)
                time.sleep(sleep)
            return self.steps


    def __init__(
            self,
            interaction: RunnerInteraction | None = None,
    ):
        tasks = [
            self.TaskTemplate("Task1", self.on_task_progress_update),
            self.TaskTemplate("Task2", self.on_task_progress_update)
        ]
        info = OperationInfo(
            name="Dev Debug",
            description="Dev Debug operation",
        )
        super().__init__(tasks, info, interaction)


    def run(self, /):
        self.execute()

    def stop(self):
        pass


from PySide6.QtWidgets import QStatusBar, QProgressBar


class QtLizStatusBar(QStatusBar):

    def __init__(self, parent):
        super().__init__(parent)
        self.showMessage("Ready")
        self.progress_bar = None
        self.__setup_status_bar()

    def __setup_status_bar(self):
        self.showMessage("Ready")
        self.progress_bar = QProgressBar()
        self.progress_bar.setRange(0, 100)
        self.progress_bar.setValue(0)
        self.progress_bar.setVisible(False)
        self.addPermanentWidget(self.progress_bar)

    def update_progress_bar(self, value: int):
        self.progress_bar.setValue(value)
        if value == 100:
            self.progress_bar.setVisible(False)
        else:
            self.progress_bar.setVisible(True)

    def set_status_message(self, message: str):
        self.showMessage(message)


import time
from abc import abstractmethod
from dataclasses import dataclass
from enum import Enum
from typing import Protocol, TypeVar, Callable, Any

from PySide6.QtCore import QThreadPool, QRunnable

from pylizlib.core.data.gen import gen_random_string
from pylizlib.core.handler.progress import QueueProgressMode, QueueProgress
from pylizlib.core.log.pylizLogger import logger

T = TypeVar("T")


@dataclass
class OperationInfo:
    name: str
    description: str


class OperationStatus(Enum):
    Pending = "Pending"
    InProgress = "In Progress"
    Completed = "Completed"
    Failed = "Failed"


class RunnerInteraction(Protocol):
    def on_runner_start(self): ...
    def on_runner_finish(self, statistics: 'RunnerStatistics'): ...
    def on_runner_stop(self): ...
    def on_runner_update_progress(self, progress: int): ...

    def on_op_start(self): ...
    def on_op_update(self, operation: Any): ...
    def on_op_update_status(self, operation_id: str, status: OperationStatus): ...
    def on_op_update_progress(self, operation_id: str, progress: int): ...
    def on_op_eta_update(self, operation_id: str, eta: str): ...
    def on_op_failed(self, operation_id: str, error: str): ...
    def on_op_finished(self, operation: Any): ...

    def on_task_start(self, task_name: str): ...
    def on_task_update_status(self, task_name: str, status: OperationStatus): ...
    def on_task_update_progress(self, task_name: str, progress: int): ...
    def on_task_failed(self, task_name: str, error: str): ...
    def on_task_finished(self, task_name: str): ...


class Task:

    def __init__(
            self,
            name: str,
            on_progress_changed: Callable[[str, int], None],
            abort_all_on_error: bool = True,
            interaction: RunnerInteraction | None = None
    ):
        self.interaction = interaction
        self.name = name
        self.abort_all_on_error = abort_all_on_error
        self.status = OperationStatus.Pending
        self.on_progress_changed = on_progress_changed
        self.result: Any = None
        self.progress = 0

    def execute(self):
        return None

    def update_task_status(self, status: OperationStatus):
        logger.debug("Updating task \"%s\" status: %s", self.name, status)
        self.status = status
        self.interaction.on_task_update_status(self.name, status) if self.interaction else None

    def update_task_progress(self, progress: int):
        logger.debug("Updating task \"%s\" progress: %s", self.name, progress)
        self.progress = progress
        self.interaction.on_task_update_progress(self.name, progress) if self.interaction else None
        self.on_progress_changed(self.name, progress)


class Operation(QRunnable):

    def __init__(
            self,
            tasks: list[Task],
            op_info: OperationInfo,
            interaction: RunnerInteraction | None = None,
    ):
        super().__init__()
        self.id = gen_random_string(10)
        self.info = op_info
        self.status = OperationStatus.Pending

        self.tasks = tasks
        self.progress = 0
        self.running = False
        self.error = None
        self.progress_obj = QueueProgress(QueueProgressMode.SINGLE, len(tasks))
        self.interaction = interaction
        self.finished_callback: Callable | None = None
        self.op_progress_update_callback: Callable | None = None
        self.current_task: Task | None = None

        self.time_started = None
        self.time_elapsed = 0
        self.time_finished = None
        self.time_estimated_total = 0
        self.time_estimated_remaining = 0

        for task in tasks:
            self.progress_obj.add_single(task.name)


    def execute_tasks(self):
        for task in self.tasks:
            try:
                self.interaction.on_task_start(task.name) if self.interaction else None
                task.update_task_status(OperationStatus.InProgress)
                logger.debug("Executing task: %s", task.name)
                self.current_task = task
                result = task.execute()
                task.result = result
                task.update_task_status(OperationStatus.Completed)
            except Exception as e:
                task.update_task_status(OperationStatus.Failed)
                logger.error("Error in task %s: %s", task.name, e)
                self.interaction.on_task_failed(task.name, str(e)) if self.interaction else None
                if task.abort_all_on_error:
                    raise RuntimeError(f"Task {task.name} failed: {e}")
            finally:
                self.interaction.on_task_finished(task.name) if self.interaction else None
                self.current_task = None


    def execute(self):
        try:
            self.set_operation_started()
            self.update_op_status(OperationStatus.InProgress)
            self.execute_tasks()
            self.update_op_status(OperationStatus.Completed)
        except Exception as e:
            self.update_op_status(OperationStatus.Failed)
            self.interaction.on_op_failed(self.id, str(e)) if self.interaction else None
            self.error = str(e)
            logger.error("Error in operation: %s", e)
        finally:
            self.set_operation_finished()


    def on_task_progress_update(self, task_name: str, progress: int):
        self.progress_obj.set_single_progress(task_name, progress)
        self.update_op_progress(self.progress_obj.get_total_progress())


    @abstractmethod
    def stop(self):
        pass

    def get_tasks_ids(self) -> list[str]:
        return [task.name for task in self.tasks]

    def get_task_results(self) -> list[Any]:
        return [task.result for task in self.tasks]

    def update_op_status(self, status: OperationStatus):
        logger.debug("Updating operation status: %s", status)
        self.status = status
        self.__update_times()
        self.interaction.on_op_update_status(self.id, status) if self.interaction else None
        self.interaction.on_op_update(self) if self.interaction else None

    def update_op_progress(self, progress: int):
        logger.debug("Updating operation progress: %s", progress)
        self.progress = progress
        self.__update_times()
        self.interaction.on_op_update_progress(self.id, progress) if self.interaction else None
        self.op_progress_update_callback(self.id, self.progress) if self.op_progress_update_callback else None
        self.interaction.on_op_update(self) if self.interaction else None

    def __update_times(self):
        if self.running:
            self.time_elapsed = time.perf_counter() - self.time_started
            if self.progress > 0:
                self.time_estimated_total = self.time_elapsed / (self.progress / 100)
                self.time_estimated_remaining = max(0, self.time_estimated_total - self.time_elapsed)
                self.interaction.on_op_eta_update(self.id, self.get_eta_formatted()) if self.interaction else None
        else:
            self.time_elapsed = self.time_finished - self.time_started

    def set_finished_callback(self, callback: Callable):
        self.finished_callback = callback

    def set_op_progress_callback(self, callback: Callable):
        self.op_progress_update_callback = callback

    def set_operation_started(self):
        logger.info("Starting operation %s", self.id)
        self.running = True
        self.interaction.on_op_start() if self.interaction else None
        self.time_started = time.perf_counter()
        self.__update_times()

    def set_operation_finished(self):
        logger.info("Finishing operation %s", self.id)
        self.running = False
        if self.finished_callback:
            self.finished_callback()
        self.interaction.on_op_finished(self) if self.interaction else None
        self.time_finished = time.perf_counter()

    def get_elapsed_formatted(self) -> str:
        """Restituisce il tempo trascorso nel formato mm:ss"""
        minutes = self.time_elapsed // 60
        seconds = self.time_elapsed % 60
        return f"{minutes:02}:{seconds:02}"

    def get_eta_formatted(self) -> str:
        """
        Restituisce il tempo stimato rimanente nel formato mm:ss.
        """
        if self.progress <= 0:
            return "--:--"

        minutes = int(self.time_estimated_remaining) // 60
        seconds = int(self.time_estimated_remaining) % 60
        return f"{minutes:02}:{seconds:02}"

    def is_failed(self):
        return self.status == OperationStatus.Failed

    def is_in_progress(self):
        return self.status == OperationStatus.InProgress

    def is_completed(self):
        return self.status == OperationStatus.Completed

    def is_pending(self):
        return self.status == OperationStatus.Pending


class RunnerStatistics:

    def __init__(self, operations: list[Operation]):
        self.operations = operations
        self.total_operations = len(operations)
        self.completed_operations = 0
        self.failed_operations = 0
        self.pending_operations = 0
        self.total_progress = 0

        for operation in operations:
            if operation.is_completed():
                self.completed_operations += 1
            elif operation.is_failed():
                self.failed_operations += 1
            elif operation.is_in_progress():
                self.total_progress += operation.progress
            elif operation.is_pending():
                self.pending_operations += 1

    def has_ops_failed(self):
        return self.failed_operations > 0

    def get_first_error(self):
        for operation in self.operations:
            if operation.is_failed():
                return operation.error
        return None


class OperationRunner:

    def __init__(
            self,
            interaction: RunnerInteraction,
            max_threads: int = 1,
            on_runner_finished: Callable | None = None,
            abort_all_on_error: bool = False,
    ):
        self.interaction = interaction
        self.max_threads = max_threads
        self.thread_pool = QThreadPool.globalInstance()
        self.thread_pool.setMaxThreadCount(self.max_threads)
        self.operation_pool: list[Operation] = []
        self.active_operations = 0
        self.progress_obj: QueueProgress | None = None
        self.abort_all_on_error = abort_all_on_error
        self.on_runner_finished = on_runner_finished


    def add(self, operation: Operation):
        self.operation_pool.append(operation)

    def start(self):
        self.interaction.on_runner_start()
        self.progress_obj = QueueProgress(QueueProgressMode.SINGLE, len(self.operation_pool))
        for op in self.operation_pool:
            self.progress_obj.add_single(op.id)
        for op in self.operation_pool:
            self.__start_next_operation()

    def stop(self):
        self.interaction.on_runner_stop()
        self.thread_pool.waitForDone()
        self.active_operations = 0
        self.operation_pool.clear()

    def __start_next_operation(self):
        can_start = self.active_operations < self.thread_pool.maxThreadCount()
        if can_start and self.operation_pool:
            op = self.operation_pool.pop(0)
            op.set_finished_callback(lambda: self.on_operation_finished(op))
            op.set_op_progress_callback(self.on_op_progress_update)
            self.thread_pool.start(op)
            self.active_operations += 1

    def on_operation_finished(self, operation: Operation):
        if self.abort_all_on_error and operation.is_failed():
            logger.error("Operation %s failed, stopping all operations", operation.id)
            self.__set_runner_finished()
            return
        self.active_operations -= 1
        self.__start_next_operation()
        if self.active_operations == 0 and not self.operation_pool:
            self.__set_runner_finished()

    def __set_runner_finished(self):
        time.sleep(1)
        statistics = RunnerStatistics(self.operation_pool)
        self.interaction.on_runner_finish(statistics)
        self.on_runner_finished() if self.on_runner_finished else None

    def on_op_progress_update(self, op_id: str, op_progress: int):
        self.progress_obj.set_single_progress(op_id, op_progress)
        self.interaction.on_runner_update_progress(self.progress_obj.get_total_progress())
